﻿// -----------------------------------------------------------------------
// <copyright file="IInclude.cs" company="Project Contributors">
// Copyright Project Contributors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// </copyright>
// -----------------------------------------------------------------------
using System;

namespace MicroLite
{
    /// <summary>
    /// The interface for including a single result.
    /// </summary>
    /// <typeparam name="T">The type of object to be included.</typeparam>
    public interface IInclude<T>
    {
        /// <summary>
        /// Gets a value indicating whether this include has a value.
        /// </summary>
        bool HasValue { get; }

        /// <summary>
        /// Gets the included value.
        /// </summary>
        /// <value>
        /// Value will be in one of the following states:
        ///  - If the overall query has not been executed the value will be equal to its default value (null for reference types).
        ///  - If the query yielded no results, it will be its default value; otherwise it will be the result of the query.
        /// </value>
        T Value { get; }

        /// <summary>
        /// Called when the included value is loaded.
        /// </summary>
        /// <param name="action">The action to be invoked.</param>
        void OnLoad(Action<IInclude<T>> action);
    }
}
